from sklearn.ensemble import RandomForestRegressor
from sklearn.linear_model import OrthogonalMatchingPursuit
from sklearn.base import BaseEstimator


class OmpForestRegressor(BaseEstimator):

    def __init__(self, models_parameters):
        self._regressor = RandomForestRegressor(n_estimators=models_parameters.forest_size,
            random_state=models_parameters.seed)
        self._models_parameters = models_parameters

    def fit(self, X_train, y_train):
        self._forest = self._train_forest(X_train, y_train)

        self._weights = self._extract_subforest(X_train, y_train)

        return self

    @property
    def forest(self):
        return self._forest

    @property
    def weights(self):
        return self._weights

    @property
    def models_parameters(self):
        return self._models_parameters

    def _train_forest(self, X_train, y_train):
        self._regressor.fit(X_train, y_train)
        forest = self._regressor.estimators_
        return forest
    
    def _extract_subforest(self, X_train, y_train):
        D = [[tree.predict([elem])[0] for tree in self._forest] for elem in X_train]
        omp = OrthogonalMatchingPursuit(
            n_nonzero_coefs=self._models_parameters.extracted_forest_size,
            fit_intercept=False, normalize=False)
        omp.fit(D, y_train)
        weights = omp.coef_
        return weights
