from bolsonaro.error_handling.logger_factory import LoggerFactory
from . import LOG_PATH

import pickle
import os
import time
import datetime


class Trainer(object):

    def __init__(self, dataset):
        self._dataset = dataset
        self._logger = LoggerFactory.create(LOG_PATH, __name__)

    def iterate(self, model, models_dir):
        self._logger.info('Training model using train set...')
        begin_time = time.time()
        model.fit(self._dataset.X_train, self._dataset.y_train)
        end_time = time.time()

        output_file_path = models_dir + os.sep + 'model.pickle'
        self._logger.info('Saving trained model to {}'.format(output_file_path))
        with open(output_file_path, 'wb') as output_file:
            pickle.dump({
                'forest': model.forest,
                'weights': model.weights,
                'training_time': end_time - begin_time,
                'datetime': datetime.datetime.now()
            }, output_file)
